@@ -2,12 +2,12 @@
  * @file osso-hw.c
  * This file implements the device signal handler functions.
  * 
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,7 @@
 
 #include "osso-internal.h"
 #include "osso-hw.h"
+#include "osso-mem.h"
 
 #define MCE_SERVICE			"com.nokia.mce"
 #define MCE_REQUEST_PATH		"/com/nokia/mce/request"
@@ -168,9 +169,10 @@ osso_return_t osso_hw_set_event_cb(osso_context_t *osso,
             _msg_handler_set_cb_f(osso, USER_LOWMEM_ON_SIGNAL_IF,
                                   lowmem_signal_handler, NULL, FALSE);
         }
-        /* FIXME? callback cannot be called now because we
-         * don't know the state */
         osso->hw_cbs.memory_low_ind.set = TRUE;
+
+        osso->hw_state.memory_low_ind = osso_mem_in_lowmem_state();
+        call_cb = TRUE;
     }
     if (state->save_unsaved_data_ind) {
         osso->hw_cbs.save_unsaved_data_ind.cb = cb;
@@ -388,7 +390,7 @@ static DBusHandlerResult signal_handler(osso_context_t *osso,
             (osso->hw_cbs.save_unsaved_data_ind.cb)(&osso->hw_state,
                 osso->hw_cbs.save_unsaved_data_ind.data);
             osso->hw_state.save_unsaved_data_ind = FALSE;
-            if (osso->autosave != NULL) {
+            if (osso->autosave.func != NULL) {
 	        /* call force autosave function */
 	        osso_application_autosave_force(osso);
             }
@@ -405,7 +407,7 @@ static DBusHandlerResult signal_handler(osso_context_t *osso,
             return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
         }
 
-        new_state = dbus_message_iter_get_boolean(&i);
+        dbus_message_iter_get_basic (&i, &new_state);
         if (osso->hw_state.system_inactivity_ind != new_state) {
             osso->hw_state.system_inactivity_ind = new_state;
             if (osso->hw_cbs.system_inactivity_ind.set) {
@@ -425,7 +427,7 @@ static DBusHandlerResult signal_handler(osso_context_t *osso,
             return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
         }
 
-        s = dbus_message_iter_get_string(&i);
+        dbus_message_iter_get_basic (&i, &s);
         if (strncmp(s, NORMAL_MODE, strlen(NORMAL_MODE)) == 0) {
             osso->hw_state.sig_device_mode_ind = OSSO_DEVMODE_NORMAL;
             write_device_state_to_file(NORMAL_MODE);
