@@ -2,12 +2,12 @@
  * @file osso-init.c
  * This file implements all initialisation and shutdown of the library.
  * 
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +24,47 @@
 #include "osso-init.h"
 #include "osso-log.h"
 
+/* for internal use only
+ * This function strdups application name and makes it
+ * suitable for being part of an object path, or returns NULL.
+ * Currently needed for allowing '.' in application name. */
+gchar* appname_to_valid_path_component(const gchar *application)
+{
+    gchar* copy = NULL, *p = NULL;
+    g_assert(application != NULL);
+    copy = g_strdup(application);
+    if (copy == NULL) {
+       return NULL;
+    }
+    for (p = g_strrstr(copy, "."); p != NULL; p = g_strrstr(p + 1, ".")) {
+        *p = '/';
+    }
+    return copy;
+}
+
+gboolean __attribute__ ((visibility("hidden")))
+validate_appname(const gchar *application)
+{
+    if (application == NULL || strstr(application, "/") != NULL) {
+	return FALSE;
+    }
+    return TRUE;
+}
+
+gboolean __attribute__ ((visibility("hidden")))
+validate_osso_context(const osso_context_t * osso)
+{
+    if (osso == NULL || !validate_appname(osso->application) ||
+        osso->version[0] == '\0') {
+       return FALSE;
+    }
+    if (strstr(osso->version, "/") != NULL) {
+       return FALSE;
+    }
+    return TRUE;
+}
+
+
 /************************************************************************/
 osso_context_t * osso_initialize(const gchar *application,
 				 const gchar *version,
@@ -31,11 +72,8 @@ osso_context_t * osso_initialize(const gchar *application,
 				 GMainContext *context)
 {
     osso_context_t *osso;
+    ULOG_DEBUG_F("application '%s', version '%s'", application, version);
 
-    dprint("Initialising application '%s' version '%s' "
-	   "%s activation, with context = %p",application, version,
-	   activation?"with":"without",context);
-    
     osso = _init(application, version);
     if (osso == NULL) {
         ULOG_CRIT_F("initialization failed: out of memory");
@@ -51,39 +89,25 @@ osso_context_t * osso_initialize(const gchar *application,
 					   (GLogFunc)_osso_log_handler,
 					   (gpointer)application);
 #endif
+
     if (activation) {
-        dprint("connecting to the activation bus");
-        ULOG_WARN_F("WARNING: if the system bus activated this program, "
-                    "Libosso does not connect to the session bus!");
-        dprint("WARNING: if the system bus activated this program, "
-               "Libosso does not connect to the session bus!");
-        fprintf(stderr, "osso_initialize() WARNING: if the system bus "
-          "activated this program, Libosso does not connect"
-          " to the session bus!\n");
-        osso->conn = _dbus_connect_and_setup(osso, DBUS_BUS_ACTIVATION,
-                                             context);
-    } else {
-        dprint("connecting to the session bus");
-        osso->conn = _dbus_connect_and_setup(osso, DBUS_BUS_SESSION,
-                                             context);
+        ULOG_WARN_F("connecting to both D-BUS busses, 'activation' "
+                    "argument does not have any effect");
     }
+    dprint("connecting to the session bus");
+    osso->conn = _dbus_connect_and_setup(osso, DBUS_BUS_SESSION, context);
     if (osso->conn == NULL) {
-        if (activation) {
-	    ULOG_CRIT_F("connecting to the activation bus failed");
-	    dprint("connecting to the activation bus failed");
-        } else {
-	    ULOG_CRIT_F("connecting to the session bus failed");
-	    dprint("connecting to the session bus failed");
-        }
-	_deinit(osso);
-	return NULL;
+        ULOG_CRIT_F("connecting to the session bus failed");
+        dprint("connecting to the session bus failed");
+        _deinit(osso);
+        return NULL;
     }
     dprint("connecting to the system bus");
     osso->sys_conn = _dbus_connect_and_setup(osso, DBUS_BUS_SYSTEM, context);
     if (osso->sys_conn == NULL) {
         ULOG_CRIT_F("connecting to the system bus failed");
         dprint("connecting to the system bus failed");
-	_deinit(osso);
+        _deinit(osso);
         return NULL;
     }
     osso->cur_conn = NULL;
@@ -105,18 +129,17 @@ void osso_deinitialize(osso_context_t *osso)
 
 
 /************************************************************************/
+
 static gboolean _validate(const gchar *application, const gchar* version)
 {
-    if ((application  == NULL) ||
-	(version == NULL)) {
+    if (application == NULL || version == NULL) {
 	return FALSE;
     }
-    if ((strstr(application, "/") != NULL) ||
-	(strstr(application, ".") != NULL)) {
+    if (!validate_appname(application)) {
 	return FALSE;
     }
-    if ((strstr(version, "/") != NULL) ||
-	(strstr(version, "/") != NULL)) {
+    if (strchr(version, '/') != NULL) {
+        ULOG_ERR_F("invalid version string '%s'", version);
 	return FALSE;
     }
     return TRUE;
@@ -127,61 +150,34 @@ static osso_context_t * _init(const gchar *application, const gchar *version)
 {
     osso_context_t * osso;
     
-    if(!_validate(application, version)) return NULL;
+    if (!_validate(application, version)) {
+	ULOG_ERR_F("invalid arguments");
+	return NULL;
+    }
 
     osso = (osso_context_t *)calloc(1, sizeof(osso_context_t));
-    if(osso == NULL) {
+    if (osso == NULL) {
+	ULOG_ERR_F("calloc failed");
 	return NULL;
     }	
 
-    osso->application = g_strdup(application);
-    if(osso->application == NULL) {
-	goto register_error0;
-    }
-    osso->version = g_strdup(version);
-    if(osso->version == NULL) {
-	goto register_error1;
-    }
-
-    osso->mime = (_osso_mime_t *)calloc(1, sizeof(_osso_mime_t));
+    g_snprintf(&osso->application[0], MAX_APP_NAME_LEN, "%s", application);
+    g_snprintf(&osso->version[0], MAX_VERSION_LEN, "%s", version);
 
     osso->ifs = g_array_new(FALSE, FALSE, sizeof(_osso_interface_t));
     osso->cp_plugins = g_array_new(FALSE, FALSE, sizeof(_osso_cp_plugin_t));
     osso->rpc_timeout = -1;
     return osso;
-
-    /**** ERROR HANDLING ****/
-    
-    register_error1:
-    if (osso->application != NULL) {
-      g_free(osso->application);
-    }
-    register_error0:
-
-    free(osso);
-        
-    return NULL;
 }
 
 /*************************************************************************/
 
 static void _deinit(osso_context_t *osso)
 {
-    if(osso == NULL)
+    if (osso == NULL) {
 	return;
-    if (osso->application != NULL) {
-	g_free(osso->application);
-	osso->application = NULL;
     }
-    if (osso->version != NULL) {
-	g_free(osso->version);
-	osso->version = NULL;
-    }
-    if (osso->object_path != NULL) {
-	g_free(osso->object_path);
-	osso->object_path = NULL;
-    }
-    if(osso->ifs != NULL) {
+    if (osso->ifs != NULL) {
         int i;
         _osso_interface_t *elem;
         /* some members need to be freed separately */
@@ -198,23 +194,16 @@ static void _deinit(osso_context_t *osso)
             }
         }
         g_array_free(osso->ifs, TRUE);
-        osso->ifs = NULL;
     }
-    if(osso->autosave != NULL) {
-	free(osso->autosave);
-	osso->autosave = NULL;
+    if (osso->cp_plugins != NULL) {
+        g_array_free(osso->cp_plugins, TRUE);
     }
-    if(osso->mime != NULL) {
-	free(osso->mime);
-	osso->mime = NULL;
-    }
-    osso->exit.cb = NULL;
-    osso->exit.data = NULL;
     
 #ifdef LIBOSSO_DEBUG
     g_log_remove_handler(NULL, osso->log_handler);
-    osso->log_handler = 0;
+    osso->log_handler = NULL;
 #endif
+    memset(osso, 0, sizeof(osso_context_t));
     free(osso);
     osso = NULL;
 }
@@ -227,47 +216,51 @@ static DBusConnection * _dbus_connect_and_setup(osso_context_t *osso,
     DBusConnection *conn;
     DBusError err;
     DBusObjectPathVTable vtable;
-    gchar service[255], *error=NULL;
+    gchar service[MAX_SVC_LEN];
     gint i;
     
     dbus_error_init(&err);
     dprint("getting the DBUS");
     conn = dbus_bus_get(bus_type, &err);
-    if(conn == NULL) {
-	ULOG_ERR_F("Unable to connect to the D-BUS daemon: %s", err.message);
-	dprint("Unable to connect to the D-BUS daemon: %s", err.message);
-	return NULL;
+    if (conn == NULL) {
+        ULOG_ERR_F("Unable to connect to the D-BUS daemon: %s", err.message);
+        dbus_error_free(&err);
+        return NULL;
     }
     dbus_connection_setup_with_g_main(conn, context);
     
     dprint("connection to the D-BUS daemon was a success");
-    
-    g_snprintf(service, 255, "%s.%s", OSSO_BUS_ROOT, osso->application);
+   
+    if (g_strrstr(osso->application, ".") != NULL) {
+        g_snprintf(service, MAX_SVC_LEN, "%s", osso->application);
+    } else {
+        g_snprintf(service, MAX_SVC_LEN, OSSO_BUS_ROOT ".%s",
+                   osso->application);
+    }
     dprint("service='%s'",service);
 
-    i = dbus_bus_acquire_service(conn, service, 
-				 0, /* undocumented parameter, flags */
-				 &err);
-    dprint("acquire service returned '%d'",i);
-    if(i <= 0) {
-	if(i == 0) {
-	    ULOG_ERR_F("Unable to acquire service '%s': Invalid parameters\n", 
-		     service);
-	}
-	else {
-	    ULOG_ERR_F("Unable to acquire service '%s': %s\n",
-		     service, err.message);
-	}
-	error = "Error: Not enough memory";
+    i = dbus_bus_request_name(conn, service,
+                              DBUS_NAME_FLAG_ALLOW_REPLACEMENT, &err);
+    dprint("acquire service returned '%d'", i);
+    if (i == -1) {
+        ULOG_ERR_F("dbus_bus_request_name failed: %s", err.message);
 	dbus_error_free(&err);
 	goto dbus_conn_error1;
     }
     
-    if (osso->object_path == NULL) {
-        osso->object_path = g_strdup_printf("/com/nokia/%s",
-                                            osso->application);
-        if (osso->object_path == NULL)
+    if (osso->object_path[0] == '\0') {
+        char* copy = NULL;
+        copy = appname_to_valid_path_component(osso->application);
+        if (copy == NULL) {
             goto dbus_conn_error1;
+        }
+        if (g_strrstr(osso->application, ".") != NULL) {
+            g_snprintf(&osso->object_path[0], MAX_OP_LEN, "/%s", copy);
+        } else {
+            g_snprintf(&osso->object_path[0], MAX_OP_LEN,
+                       OSSO_BUS_ROOT_PATH "/%s", copy);
+        }
+        g_free(copy);
     }
     dprint("osso->object_path='%s'", osso->object_path);
 
@@ -286,26 +279,22 @@ static DBusConnection * _dbus_connect_and_setup(osso_context_t *osso,
     dprint("adding Filter function %p",&_debug_filter);
     if(!dbus_connection_add_filter(conn, &_debug_filter, NULL, NULL))
     {
-	error = "Error: unable to add debug filter";
+        ULOG_ERR_F("dbus_connection_add_filter failed");
 	goto dbus_conn_error3;
     }
 #endif
     dprint("adding Filter function %p",&_msg_handler);
     if(!dbus_connection_add_filter(conn, &_msg_handler, osso, NULL))
     {
-	error = "Error: unable to add _msg_handler as a filter";
+        ULOG_ERR_F("dbus_connection_add_filter failed");
 	goto dbus_conn_error4;
     }
-    dprint("My base service is '%s'",
-	   dbus_bus_get_base_service(conn));
-
-    dbus_error_free(&err);
+    dprint("My base service is '%s'", dbus_bus_get_unique_name(conn));
 
     return conn;
 
     /**** ERROR HANDLING ****/
 
-/*    dbus_connection_remove_filter(conn, _msg_handler, osso); */
     dbus_conn_error4:
 #ifdef LIBOSSO_DEBUG
     dbus_connection_remove_filter(conn, _debug_filter, NULL);
@@ -314,14 +303,10 @@ static DBusConnection * _dbus_connect_and_setup(osso_context_t *osso,
     dbus_connection_unregister_object_path(conn, osso->object_path);
 
     dbus_conn_error2:
-    g_free(osso->object_path);
     dbus_conn_error1:
         
     /* no explicit disconnection, because the connections are shared */
     dbus_connection_unref(conn);
-    if(error != NULL) {
-	ULOG_ERR_F("%s", error);
-    }
 
     return NULL;
 }
