@@ -2,12 +2,12 @@
  * @file osso-internal.h
  * This file is the private include for all functions.
  * 
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,6 +58,14 @@
 
 # define OSSO_RPC_REPLY_TIMEOUT -1
 
+/* DBus interface, service, and object path maximum lengths */
+#define MAX_IF_LEN 255
+#define MAX_SVC_LEN 255
+#define MAX_OP_LEN 255
+
+#define MAX_APP_NAME_LEN 50
+#define MAX_VERSION_LEN 30
+
 typedef DBusHandlerResult (_osso_interface_cb_f)(osso_context_t *osso,
 						 DBusMessage *msg,
 						 gpointer data);
@@ -117,24 +125,23 @@ struct osso_af_context_t {
     DBusConnection *conn;
     DBusConnection *sys_conn;
     DBusConnection *cur_conn;
-    gchar *application;
-    gchar *version;
-    gchar *object_path;
+    gchar application[MAX_APP_NAME_LEN + 1];
+    gchar version[MAX_VERSION_LEN + 1];
+    char object_path[MAX_OP_LEN + 1];
     GArray *ifs;
-    _osso_autosave_t *autosave;
+    _osso_autosave_t autosave;
     guint log_handler;
     _osso_hw_cb_t hw_cbs;
     osso_hw_state_t hw_state;
-    int environment_set;
     guint rpc_timeout;
-    _osso_mime_t *mime;
+    _osso_mime_t mime;
     GArray *cp_plugins;
     _osso_exit_t exit;
 };
 
 # ifdef LIBOSSO_DEBUG
-#  define dprint(f, a...) fprintf(stderr, "%s:%d %s(): "f"\n", __FILE__, \
-				     __LINE__, __func__, ##a)
+#  define dprint(f, a...) (fprintf(stderr, "%s:%d %s(): "f"\n", __FILE__, \
+				     __LINE__, __func__, ##a),fflush(stderr))
 # else
 #  define dprint(f, a...)
 # endif /* LIBOSSO_DEBUG */
@@ -153,17 +160,15 @@ DBusHandlerResult _msg_handler(DBusConnection *conn, DBusMessage *msg,
 void _msg_handler_set_cb_f(osso_context_t *osso, const gchar *interface,
 			   _osso_interface_cb_f *cb, gpointer data, 
 			   gboolean method);
+void _msg_handler_set_cb_f_free_data(osso_context_t *osso, const gchar *interface,
+				     _osso_interface_cb_f *cb, gpointer data, 
+				     gboolean method);
 gpointer _msg_handler_rm_cb_f(osso_context_t *osso, const gchar *interface,
 			      _osso_interface_cb_f *cb, gboolean method);
 void _msg_handler_set_ret(osso_context_t *osso, gint serial,
 			  osso_rpc_t *retval);
 void _msg_handler_rm_ret(osso_context_t *osso, gint serial);
 
-osso_return_t _rpc_run (osso_context_t * osso, DBusConnection * conn,
-                        const gchar * service, const gchar * object_path,
-                        const gchar * interface, const gchar * method,
-                        osso_rpc_t * retval, int first_arg_type,
-                        va_list var_args);
 /* this is only needed by some unit testing code */
 osso_return_t _test_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
                                 const gchar *object_path,
@@ -171,6 +176,19 @@ osso_return_t _test_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
 			        osso_rpc_cb_f *cb, gpointer data,
 			        gboolean use_system_bus);
 
+gchar* appname_to_valid_path_component(const gchar *application);
+gboolean validate_appname(const gchar *application);
+
+/**
+ * This internal function performs a simple validation for the application
+ * and version information of the osso_context regarding their validity
+ * as components of the filesystem (no slashes, value not NULL etc)
+ * @param osso The osso context containing the application name and
+ * version information
+ * @return TRUE if the context passes the validation, FALSE otherwise.
+ */
+gboolean validate_osso_context(const osso_context_t * osso);
+
 #define _launch_app(osso, application) \
     osso_rpc_run_with_defaults(osso, application, \
 				   OSSO_BUS_ACTIVATE, NULL, NULL);
