@@ -4,8 +4,8 @@
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,10 +18,11 @@
  *
  */
                 
-#include <osso-internal.h>
 #include <stdarg.h>
 #include <syslog.h>
 
+#include "osso-internal.h"
+#include "log-functions.h"
 
 void osso_log(int level, const char *format, ...) 
 {
