@@ -2,12 +2,12 @@
  * @file osso-rpc.c
  * This file implements rpc calls from one application to another.
  * 
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,11 +19,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#define TASK_NAV_SERVICE                    "com.nokia.tasknav"
-/* NOTICE: Keep these in sync with values in hildon-navigator/windowmanager.c! */
-#define APP_LAUNCH_BANNER_METHOD_INTERFACE  "com.nokia.tasknav.app_launch_banner"
-#define APP_LAUNCH_BANNER_METHOD_PATH       "/com/nokia/tasknav/app_launch_banner"
-#define APP_LAUNCH_BANNER_METHOD            "app_launch_banner"
+#define TASK_NAV_SERVICE "com.nokia.tasknav"
+/* NOTICE: Keep these in sync with values in
+ * hildon-navigator/windowmanager.c! */
+#define APP_LAUNCH_BANNER_METHOD_INTERFACE \
+            "com.nokia.tasknav.app_launch_banner"
+#define APP_LAUNCH_BANNER_METHOD_PATH \
+            "/com/nokia/tasknav/app_launch_banner"
+#define APP_LAUNCH_BANNER_METHOD "app_launch_banner"
 
 
 #include "osso-internal.h"
@@ -36,9 +39,19 @@ static void _append_args(DBusMessage *msg, int type, va_list var_args);
 static void _append_arg (DBusMessage * msg, osso_rpc_t * arg);
 static void _get_arg (DBusMessageIter * iter, osso_rpc_t * retval);
 static void _async_return_handler (DBusPendingCall * pending, void *data);
+static osso_return_t _rpc_run_with_argfill (osso_context_t *osso,
+					    DBusConnection *conn,
+					    const gchar *service,
+					    const gchar *object_path,
+					    const gchar *interface,
+					    const gchar *method,
+					    osso_rpc_t *retval, 
+					    osso_rpc_argfill *argfill,
+					    void *argfill_data);
 
 typedef struct {
     osso_rpc_cb_f *func;
+    osso_rpc_retval_free_f *retval_free;
     gpointer data;
 }_osso_rpc_t;
 
@@ -50,7 +63,29 @@ typedef struct {
 }_osso_rpc_async_t;
 
 
+void osso_rpc_free_val (osso_rpc_t *rpc)
+{
+  if (rpc->type == DBUS_TYPE_STRING)
+	  g_free (rpc->value.s);
+  rpc->type = DBUS_TYPE_INVALID;
+}
+
 /************************************************************************/
+typedef struct {
+  int argument_type;
+  va_list arg_list;
+} fill_from_va_list_data;
+
+static void
+fill_from_va_list (DBusMessage *msg, void *raw)
+{
+  fill_from_va_list_data *data = (fill_from_va_list_data *)raw;
+
+  if (data->argument_type != DBUS_TYPE_INVALID) {
+    _append_args(msg, data->argument_type, data->arg_list);
+  }
+}
+
 osso_return_t osso_rpc_run(osso_context_t *osso, const gchar *service,
 			   const gchar *object_path,
 			   const gchar *interface,
@@ -58,147 +93,214 @@ osso_return_t osso_rpc_run(osso_context_t *osso, const gchar *service,
 			   int argument_type, ...)
 {
     osso_return_t ret;
-    va_list arg_list;
+    fill_from_va_list_data data;
     
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
 	(interface == NULL) || (method == NULL))
 	return OSSO_INVALID;
 
-    va_start(arg_list, argument_type);
+    data.argument_type = argument_type;
+    va_start(data.arg_list, argument_type);
     
-    ret = _rpc_run(osso, osso->conn, service, object_path, interface,
-		   method, retval, argument_type, arg_list);
-    va_end(arg_list);
+    ret = _rpc_run_with_argfill (osso, osso->conn,
+				 service, object_path, interface, method,
+				 retval,
+				 fill_from_va_list, &data);
+
+    va_end(data.arg_list);
     return ret;
 }
 
+/************************************************************************/
+osso_return_t osso_rpc_run_with_argfill (osso_context_t *osso,
+					 const gchar *service,
+					 const gchar *object_path,
+					 const gchar *interface,
+					 const gchar *method,
+					 osso_rpc_t *retval,
+					 osso_rpc_argfill *argfill,
+					 void *argfill_data)
+{
+    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
+	(interface == NULL) || (method == NULL))
+	return OSSO_INVALID;
+
+    return _rpc_run_with_argfill (osso, osso->conn,
+				  service, object_path, interface, method, 
+				  retval,
+				  argfill, argfill_data);
+}
 
 /************************************************************************/
-osso_return_t __attribute__ ((visibility("hidden")))
-_rpc_run(osso_context_t *osso, DBusConnection *conn,
-         const gchar *service, const gchar *object_path,
-         const gchar *interface, const gchar *method,
-         osso_rpc_t *retval, int first_arg_type,
-         va_list var_args)
+osso_return_t osso_rpc_run_system(osso_context_t *osso, const gchar *service,
+			   const gchar *object_path,
+			   const gchar *interface,
+			   const gchar *method, osso_rpc_t *retval,
+			   int argument_type, ...)
+{
+    osso_return_t ret;
+    fill_from_va_list_data data;
+    
+    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
+	(interface == NULL) || (method == NULL))
+	return OSSO_INVALID;
+
+    data.argument_type = argument_type;
+    va_start(data.arg_list, argument_type);
+    
+    ret = _rpc_run_with_argfill (osso, osso->sys_conn,
+				 service, object_path, interface, method,
+				 retval,
+				 fill_from_va_list, &data);
+
+    va_end(data.arg_list);
+    return ret;
+}
+
+/************************************************************************/
+osso_return_t osso_rpc_run_system_with_argfill (osso_context_t *osso,
+						const gchar *service,
+						const gchar *object_path,
+						const gchar *interface,
+						const gchar *method,
+						osso_rpc_t *retval,
+						osso_rpc_argfill *argfill,
+						void *argfill_data)
+{
+    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
+	(interface == NULL) || (method == NULL))
+	return OSSO_INVALID;
+
+    return _rpc_run_with_argfill (osso, osso->sys_conn,
+				  service, object_path, interface, method, 
+				  retval,
+				  argfill, argfill_data);
+}
+
+/************************************************************************/
+static osso_return_t
+_rpc_run_with_argfill (osso_context_t *osso, DBusConnection *conn,
+		       const gchar *service, const gchar *object_path,
+		       const gchar *interface, const gchar *method,
+		       osso_rpc_t *retval, 
+		       osso_rpc_argfill *argfill,
+		       void *argfill_data)
 {
     DBusMessage *msg;
     dbus_bool_t b;
     
-    if(conn == NULL)
+    dprint("");
+    if (conn == NULL) {
+        ULOG_ERR_F("invalid arguments");
 	return OSSO_INVALID;
+    }
     
-    dprint("New method: %s%s::%s:%s",service,object_path,interface,method);
+    dprint("New method: %s:%s:%s:%s",service,object_path,interface,method);
     msg = dbus_message_new_method_call(service, object_path,
 				       interface, method);
-    if(msg == NULL)
+    if (msg == NULL) {
+        ULOG_ERR_F("dbus_message_new_method_call failed");
 	return OSSO_ERROR;
+    }
 
-    if(first_arg_type != DBUS_TYPE_INVALID)
-	_append_args(msg, first_arg_type, var_args);
+    argfill (msg, argfill_data);
 
-    dbus_message_set_auto_activation(msg, TRUE);
+    dbus_message_set_auto_start(msg, TRUE);
     
-    if(retval == NULL) {
+    if (retval == NULL) {
 	dbus_message_set_no_reply(msg, TRUE);
 	b = dbus_connection_send(conn, msg, NULL);
+        dbus_message_unref(msg);
         if (b) {
-            dbus_connection_flush(conn);
-            dbus_message_unref(msg);
 
 	    /* Tell TaskNavigator to show "launch banner" */
-	    msg = dbus_message_new_method_call(
-			    TASK_NAV_SERVICE,
+	    msg = dbus_message_new_method_call(TASK_NAV_SERVICE,
 			    APP_LAUNCH_BANNER_METHOD_PATH,
 			    APP_LAUNCH_BANNER_METHOD_INTERFACE,
 			    APP_LAUNCH_BANNER_METHOD );
    
-	    if ( msg != NULL ) {
-		    dbus_message_append_args( msg,
-				    DBUS_TYPE_STRING, service,
-				    DBUS_TYPE_INVALID);
-
-		    b = dbus_connection_send(conn, msg, NULL);
-
-		    if (b) {
-			    dbus_connection_flush(conn);
-		    } else {
-			    dprint( "Failed to send signal to TN!" );
-		    }
-		    
-		    dbus_message_unref(msg);
+	    if (msg != NULL) {
+                dbus_message_append_args(msg, DBUS_TYPE_STRING, &service,
+                                         DBUS_TYPE_INVALID);
 
+                b = dbus_connection_send(conn, msg, NULL);
+                if (!b) {
+                    ULOG_WARN_F("dbus_connection_send failed");
+                }
+                dbus_message_unref(msg);
 	    }
-	    
-	    return OSSO_OK;
+            else {
+                ULOG_WARN_F("dbus_message_new_method_call failed");
+            }
+            return OSSO_OK;
         }
         else {
-            dbus_message_unref(msg);
+            ULOG_ERR_F("dbus_connection_send failed");
             return OSSO_ERROR;
         }
     }
     else {
         DBusError err;
-        DBusMessage *ret;
+        DBusMessage *reply = NULL;
         dbus_error_init(&err);
-	ret = dbus_connection_send_with_reply_and_block(conn, msg, 
+	reply = dbus_connection_send_with_reply_and_block(conn, msg, 
 							osso->rpc_timeout,
 							&err);
         dbus_message_unref(msg);
 
-        if (!ret) {
-            dprint("Error return: %s",err.message);
-	    retval->type = DBUS_TYPE_STRING;
-	    retval->value.s = g_strdup(err.message);
-	    dbus_error_free(&err);
+        if (reply == NULL) {
+            ULOG_ERR_F("dbus_connection_send_with_reply_and_block error: %s",
+                       err.message);
+            retval->type = DBUS_TYPE_STRING;
+            retval->value.s = g_strdup(err.message);
+            dbus_error_free(&err);
             return OSSO_RPC_ERROR;
         }
         else {
 	    DBusMessageIter iter;
-	    DBusError err;
 
-	    switch(dbus_message_get_type(ret)) {
+	    switch (dbus_message_get_type(reply)) {
 	      case DBUS_MESSAGE_TYPE_ERROR:
-		dbus_set_error_from_message(&err, ret);
+		dbus_set_error_from_message(&err, reply);
 		retval->type = DBUS_TYPE_STRING;
 		retval->value.s = g_strdup(err.message);
 		dbus_error_free(&err);
+		dbus_message_unref(reply);
 		return OSSO_RPC_ERROR;
 	      case DBUS_MESSAGE_TYPE_METHOD_RETURN:
 		dprint("message return");
-		dbus_message_iter_init(ret, &iter);
+		dbus_message_iter_init(reply, &iter);
 	    
 		_get_arg(&iter, retval);
 	    
-		dbus_message_unref(ret);
+		dbus_message_unref(reply);
 	
 		/* Tell TaskNavigator to show "launch banner" */
-		msg = dbus_message_new_method_call(
-				TASK_NAV_SERVICE,
+		msg = dbus_message_new_method_call(TASK_NAV_SERVICE,
 				APP_LAUNCH_BANNER_METHOD_PATH,
 				APP_LAUNCH_BANNER_METHOD_INTERFACE,
 				APP_LAUNCH_BANNER_METHOD );
 
-		if ( msg != NULL ) {
-			dbus_message_append_args( msg,
-					DBUS_TYPE_STRING, service,
-					DBUS_TYPE_INVALID);
+		if (msg != NULL) {
+                    dbus_message_append_args(msg, DBUS_TYPE_STRING,
+                                             &service, DBUS_TYPE_INVALID);
 
-			b = dbus_connection_send(conn, msg, NULL);
-
-			if (b) {
-				dbus_connection_flush(conn);
-			} else {
-				dprint( "Failed to send signal to TN!" );
-			}
-
-			dbus_message_unref(msg);
+                    b = dbus_connection_send(conn, msg, NULL);
 
+                    if (!b) {
+                        ULOG_WARN_F("dbus_connection_send failed");
+                    }
+                    dbus_message_unref(msg);
 		}
-
-		return OSSO_OK;
+                else {
+                    ULOG_WARN_F("dbus_message_new_method_call failed");
+                }
+                return OSSO_OK;
 	      default:
+                ULOG_WARN_F("got unknown message type as reply");
 		retval->type = DBUS_TYPE_STRING;
 		retval->value.s = g_strdup("Invalid return value");
+		dbus_message_unref(reply);
 		return OSSO_RPC_ERROR;
 	    }
         }
@@ -214,66 +316,85 @@ osso_return_t osso_rpc_run_with_defaults(osso_context_t *osso,
 					 int argument_type,
 					 ...)
 {
-    gchar service[256] = {0}, object_path[256] = {0}, interface[256] = {0};
-    va_list arg_list;
+    char service[MAX_SVC_LEN] = {0}, object_path[MAX_OP_LEN] = {0},
+         interface[MAX_IF_LEN] = {0};
+    fill_from_va_list_data data;
     osso_return_t ret;
+    gchar* copy = NULL;
 
+    dprint("");
     if( (osso == NULL) || (application == NULL) || (method == NULL) )
 	return OSSO_INVALID;
     
-    g_snprintf(service, 255, "%s.%s", OSSO_BUS_ROOT, application);
-    g_snprintf(object_path, 255, "%s/%s", OSSO_BUS_ROOT_PATH,
-	       application);
-    g_snprintf(interface, 255, "%s", service);
+    g_snprintf(service, MAX_SVC_LEN, OSSO_BUS_ROOT ".%s", application);
+    copy = appname_to_valid_path_component(application);
+    if (copy == NULL) {
+	ULOG_ERR_F("failure in appname_to_valid_path_component");
+	return OSSO_ERROR;
+    }
+    g_snprintf(object_path, MAX_OP_LEN, OSSO_BUS_ROOT_PATH "/%s", copy);
+    g_free(copy); copy = NULL;
+    g_snprintf(interface, MAX_IF_LEN, "%s", service);
 
-    va_start(arg_list, argument_type);
+    data.argument_type = argument_type;
+    va_start(data.arg_list, argument_type);
 
-    ret = _rpc_run(osso, osso->conn, service, object_path, interface,
-		   method, retval, argument_type, arg_list);
+    ret = _rpc_run_with_argfill (osso, osso->conn,
+				 service, object_path, interface, method,
+				 retval,
+				 fill_from_va_list, &data);
     
-    va_end(arg_list);
+    va_end(data.arg_list);
 
     return ret;
 }
 
 /************************************************************************/
-static osso_return_t _rpc_async_run(osso_context_t *osso,
-			     const gchar *service,
-			     const gchar *object_path,
-			     const gchar *interface,
-			     const gchar *method,
-			     osso_rpc_async_f *async_cb,
-			     gpointer data, int first_arg_type,
-			     va_list var_args)
+osso_return_t osso_rpc_async_run_with_argfill (osso_context_t *osso,
+						  const gchar *service,
+						  const gchar *object_path,
+						  const gchar *interface,
+						  const gchar *method,
+						  osso_rpc_async_f *async_cb,
+						  gpointer data,
+						  osso_rpc_argfill *argfill,
+						  void *argfill_data)
 {
-    DBusMessage *msg;
-    DBusPendingCall *pending;
-    dbus_bool_t b;
+    DBusMessage *msg = NULL;
+    DBusPendingCall *pending = NULL;
+    dbus_bool_t succ = FALSE;
     _osso_rpc_async_t *rpc = NULL;
    
-    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
-	return OSSO_INVALID;
+    if (osso == NULL || service == NULL || object_path == NULL ||
+        interface == NULL || method == NULL) {
+        ULOG_ERR_F("invalid arguments");
+        return OSSO_INVALID;
+    }
 
-    dprint("New method: %s%s::%s:%s",service,object_path,interface,method);
+    dprint("New method: %s:%s:%s:%s",service,object_path,interface,method);
     msg = dbus_message_new_method_call(service, object_path,
 				       interface, method);
-    if(msg == NULL)
-	return OSSO_ERROR;
+    if (msg == NULL) {
+        ULOG_ERR_F("dbus_message_new_method_call failed");
+        return OSSO_ERROR;
+    }
 
-    dbus_message_set_auto_activation(msg, TRUE);
+    dbus_message_set_auto_start(msg, TRUE);
     
-    if(first_arg_type != DBUS_TYPE_INVALID)
-	_append_args(msg, first_arg_type, var_args);
-/*	dbus_message_append_args_valist(msg, first_arg_type, var_args);*/
+    argfill (msg, argfill_data);
 
-    if(async_cb == NULL) {
+    if (async_cb == NULL) {
+	dprint("no reply wanted");
 	dbus_message_set_no_reply(msg, TRUE);
-	b = dbus_connection_send(osso->conn, msg, NULL);
+	succ = dbus_connection_send(osso->conn, msg, NULL);
     }
     else {
+	dprint("caller wants reply");
 	rpc = (_osso_rpc_async_t *)calloc(1, sizeof(_osso_rpc_async_t));
-	if(rpc == NULL) return OSSO_ERROR;
+	if (rpc == NULL) {
+            ULOG_ERR_F("calloc failed");
+            return OSSO_ERROR;
+        }
 	
 	rpc->func = async_cb;
 	rpc->data = data;
@@ -285,47 +406,46 @@ static osso_return_t _rpc_async_run(osso_context_t *osso,
 	dprint("rpc->interface = '%s'",rpc->interface);
 	dprint("rpc->method = '%s'",rpc->method);
 
-	b = dbus_connection_send_with_reply(osso->conn, msg, 
+	succ = dbus_connection_send_with_reply(osso->conn, msg, 
 					    &pending, osso->rpc_timeout);
     }
 
-    if(b) {
-	if(async_cb != NULL) {
+    if (succ) {
+	dprint("succ is true");
+	if (async_cb != NULL) {
 	    dbus_pending_call_set_notify(pending,
 					 _async_return_handler,
 					 rpc, NULL);
 	}
-	dbus_connection_flush(osso->conn);
 	dbus_message_unref(msg);
 
 	/* Tell TaskNavigator to show "launch banner" */
-	msg = dbus_message_new_method_call(
-			TASK_NAV_SERVICE,
-			APP_LAUNCH_BANNER_METHOD_PATH,
-			APP_LAUNCH_BANNER_METHOD_INTERFACE,
-			APP_LAUNCH_BANNER_METHOD );
+	msg = dbus_message_new_method_call(TASK_NAV_SERVICE,
+                        APP_LAUNCH_BANNER_METHOD_PATH,
+                        APP_LAUNCH_BANNER_METHOD_INTERFACE,
+                        APP_LAUNCH_BANNER_METHOD );
 
-	if ( msg != NULL ) {
-		dbus_message_append_args( msg,
-				DBUS_TYPE_STRING, service,
-				DBUS_TYPE_INVALID);
+	if (msg != NULL) {
+            dbus_message_append_args(msg, DBUS_TYPE_STRING, &service,
+                                     DBUS_TYPE_INVALID);
 
-		b = dbus_connection_send(osso->conn, msg, NULL);
+            succ = dbus_connection_send(osso->conn, msg, NULL);
 
-		if (b) {
-			dbus_connection_flush(osso->conn);
-		} else {
-			dprint( "Failed to send signal to TN!" );
-		}
-
-		dbus_message_unref(msg);
+            if (!succ) {
+                ULOG_WARN_F("dbus_connection_send failed");
+            }
+            dbus_message_unref(msg);
 	}
+        else {
+            ULOG_WARN_F("dbus_message_new_method_call failed");
+        }
 
 	return OSSO_OK;
     }
     else {
-	dbus_message_unref(msg);
-	return OSSO_ERROR;
+        ULOG_ERR_F("dbus_connection_send(_with_reply) failed");
+        dbus_message_unref(msg);
+        return OSSO_ERROR;
     }
 }
 
@@ -339,7 +459,7 @@ osso_return_t osso_rpc_async_run(osso_context_t *osso,
 				 gpointer data,
 				 int argument_type, ...)
 {
-    va_list arg_list;
+    fill_from_va_list_data argfill_data;
     osso_return_t ret;
    
 	
@@ -347,12 +467,16 @@ osso_return_t osso_rpc_async_run(osso_context_t *osso,
 	(interface == NULL) || (method == NULL))
 	return OSSO_INVALID;
 
-    va_start(arg_list, argument_type);
+    argfill_data.argument_type = argument_type;
+    va_start(argfill_data.arg_list, argument_type);
 
-    ret = _rpc_async_run(osso, service, object_path, interface, method,
-			 async_cb, data, argument_type, arg_list);
+    ret = osso_rpc_async_run_with_argfill (osso,
+				       service, object_path, interface, method,
+				       async_cb, data,
+				       fill_from_va_list, &argfill_data);
+    dprint("osso_rpc_async_run_with_argfill returned");
     
-    va_end(arg_list);
+    va_end(argfill_data.arg_list);
     
     return ret;
 }
@@ -365,44 +489,57 @@ osso_return_t osso_rpc_async_run_with_defaults(osso_context_t *osso,
 					       gpointer data,
 					       int argument_type, ...)
 {
-    va_list arg_list;
+    fill_from_va_list_data argfill_data;
     osso_return_t ret;
-    gchar service[256] = {0}, object_path[256] = {0}, interface[256] = {0};
+    char service[MAX_SVC_LEN] = {0}, object_path[MAX_OP_LEN] = {0},
+         interface[MAX_IF_LEN] = {0};
+    gchar* copy = NULL;
 
     if( (osso == NULL) || (application == NULL) || (method == NULL) )
 	return OSSO_INVALID;
     
-    g_snprintf(service, 255, "%s.%s", OSSO_BUS_ROOT, application);
-    g_snprintf(object_path, 255, "%s/%s", OSSO_BUS_ROOT_PATH,
-	       application);
-    g_snprintf(interface, 255, "%s", service);
+    g_snprintf(service, MAX_SVC_LEN, OSSO_BUS_ROOT ".%s", application);
+    copy = appname_to_valid_path_component(application);
+    if (copy == NULL) {
+        ULOG_ERR_F("failure in appname_to_valid_path_component");
+	return OSSO_ERROR;
+    }
+    g_snprintf(object_path, MAX_OP_LEN, OSSO_BUS_ROOT_PATH "/%s", copy);
+    g_free(copy); copy = NULL;
+    g_snprintf(interface, MAX_IF_LEN, "%s", service);
 
-    va_start(arg_list, argument_type);
+    argfill_data.argument_type = argument_type;
+    va_start(argfill_data.arg_list, argument_type);
 
-    ret = _rpc_async_run(osso, service, object_path, interface, method,
-			 async_cb, data, argument_type, arg_list);
+    ret = osso_rpc_async_run_with_argfill (osso,
+				       service, object_path, interface, method,
+				       async_cb, data,
+				       fill_from_va_list, &argfill_data);
+    dprint("osso_rpc_async_run_with_argfill returned");
     
-    va_end(arg_list);
+    va_end(argfill_data.arg_list);
 
     return ret;
 }
 
 /************************************************************************/
-static osso_return_t _rpc_set_cb_f(osso_context_t *osso, const gchar *service,
-			    const gchar *object_path,
-			    const gchar *interface,
-			    osso_rpc_cb_f *cb, gpointer data,
-			    gboolean use_system_bus)
+static osso_return_t _rpc_set_cb_f(osso_context_t *osso,
+                                   const gchar *service,
+                                   const gchar *object_path,
+                                   const gchar *interface,
+                                   osso_rpc_cb_f *cb, gpointer data,
+                                   osso_rpc_retval_free_f *retval_free,
+                                   gboolean use_system_bus)
 {
     DBusError err;
     struct DBusObjectPathVTable vt;
     _osso_rpc_t *rpc;
-    gint i;
-    gboolean b;
+    int ret;
+    dbus_bool_t bret;
 
-    dprint("registering method service '%s' at '%s' on interface '%s',"
-	   " on the %sbus", service, object_path, interface,
-	   use_system_bus?"SYSTEM":"SESSION");
+    ULOG_DEBUG_F("s '%s' o '%s' i '%s', %s bus",
+                 service, object_path, interface,
+                 use_system_bus ? "system" : "session");
     
     rpc = (_osso_rpc_t *)calloc(1, sizeof(_osso_rpc_t));
     if (rpc == NULL) {
@@ -412,44 +549,46 @@ static osso_return_t _rpc_set_cb_f(osso_context_t *osso, const gchar *service,
     
     dbus_error_init(&err);
 
-    dprint("acquiring service '%s'", service);
-    i = dbus_bus_acquire_service(use_system_bus?osso->sys_conn:osso->conn,
-				 service, 0, &err);
-#ifdef LIBOSSO_DEBUG
-    if(i <= 0) {
-	if(i == 0) {
-	    dprint("Unable to acquire service '%s': Invalid parameters\n", 
-		   service);
-	}
-	else {
-	    dprint("Unable to acquire service '%s': %s\n",
-		   service, err.message);
-	}
+    dprint("requesting service '%s'", service);
+    ret = dbus_bus_request_name(use_system_bus ? osso->sys_conn :
+              osso->conn, service, DBUS_NAME_FLAG_ALLOW_REPLACEMENT |
+              DBUS_NAME_FLAG_REPLACE_EXISTING |
+              DBUS_NAME_FLAG_DO_NOT_QUEUE, &err);
+    if (ret == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER ||
+        ret == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER) {
+        /* success */ 
+    } else if (ret == DBUS_REQUEST_NAME_REPLY_IN_QUEUE ||
+               ret == DBUS_REQUEST_NAME_REPLY_EXISTS) {
+        /* this should be impossible */
+        ULOG_ERR_F("dbus_bus_request_name is broken");
+        return OSSO_ERROR;
+    } else if (ret == -1) {
+        ULOG_ERR_F("dbus_bus_request_name for '%s' failed: %s",
+                   service, err.message);
+	dbus_error_free(&err);
+        return OSSO_ERROR;
     }
-    else
-	dprint("success");
-#endif
-    dbus_error_free(&err);
 
     vt.message_function = _msg_handler;
     vt.unregister_function = NULL;
     
-    dprint("registering object_path: '%s'",object_path);
-    b = dbus_connection_register_object_path(use_system_bus?
-					     osso->sys_conn:osso->conn,
-					     object_path,
-					     &vt, (void *)osso);
-    dprint("%s",b?"success":"failure");
+    dprint("registering object_path: '%s'", object_path);
+    bret = dbus_connection_register_object_path(use_system_bus ?
+               osso->sys_conn : osso->conn, object_path, &vt,
+               (void *)osso);
+    if (!bret) {
+        ULOG_WARN_F("ignoring dbus_connection_register_object_path"
+                    " failure, maybe it was already registered");
+	/* error ignored because we cannot distinguish OOM and already
+	 * registered object path */
+    }
 
     rpc->func = cb;
+    rpc->retval_free = retval_free;
     rpc->data = data;
-    dprint("rpc = %p",rpc);
-    dprint("rpc->func = %p",rpc->func);
 
     _msg_handler_set_cb_f(osso, interface, _rpc_handler,
-			  (gpointer)rpc, TRUE);
-    dprint("rpc->func = %p",rpc->func);
-
+                          (gpointer)rpc, TRUE);
     return OSSO_OK;
 }
 
@@ -461,27 +600,36 @@ osso_return_t _test_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
                                  gboolean use_system_bus)
 {
     return _rpc_set_cb_f(osso, service, object_path, interface,
-                         cb, data, use_system_bus);
+                         cb, data, NULL, use_system_bus);
 }
 
 /************************************************************************/
-osso_return_t osso_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
-		       const gchar *object_path, const gchar *interface,
-		       osso_rpc_cb_f *cb, gpointer data)
+osso_return_t osso_rpc_set_cb_f_with_free (osso_context_t *osso, const gchar *service,
+					   const gchar *object_path, const gchar *interface,
+					   osso_rpc_cb_f *cb, gpointer data,
+					   osso_rpc_retval_free_f *retval_free)
 {
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
 	(interface == NULL) || (cb == NULL))
 	return OSSO_INVALID;
     return _rpc_set_cb_f(osso, service, object_path,
-			 interface,cb, data, FALSE);
+			 interface,cb, data, retval_free, FALSE);
+}
+
+osso_return_t osso_rpc_set_cb_f (osso_context_t *osso, const gchar *service,
+				 const gchar *object_path, const gchar *interface,
+				 osso_rpc_cb_f *cb, gpointer data)
+{
+    return osso_rpc_set_cb_f_with_free (osso, service, object_path, interface, cb, data, NULL);
 }
 
 /************************************************************************/
-osso_return_t osso_rpc_set_default_cb_f(osso_context_t *osso,
-					osso_rpc_cb_f *cb,
-					gpointer data)
+osso_return_t osso_rpc_set_default_cb_f_with_free (osso_context_t *osso,
+						   osso_rpc_cb_f *cb,
+						   gpointer data,
+						   osso_rpc_retval_free_f *retval_free)
 {
-    gchar interface[256] = {0};
+    gchar interface[MAX_IF_LEN] = {0};
     _osso_rpc_t *rpc;
     
     if( (osso == NULL) || (cb == NULL) )
@@ -493,8 +641,10 @@ osso_return_t osso_rpc_set_default_cb_f(osso_context_t *osso,
 	return OSSO_ERROR;
     }
     
-    g_snprintf(interface, 255, "%s.%s", OSSO_BUS_ROOT, osso->application);    
+    g_snprintf(interface, MAX_IF_LEN, OSSO_BUS_ROOT ".%s",
+               osso->application);    
     rpc->func = cb;
+    rpc->retval_free = retval_free;
     rpc->data = data;
     
     _msg_handler_set_cb_f_free_data(osso, interface, _rpc_handler,
@@ -502,6 +652,13 @@ osso_return_t osso_rpc_set_default_cb_f(osso_context_t *osso,
     return OSSO_OK;
 }
 
+osso_return_t osso_rpc_set_default_cb_f (osso_context_t *osso,
+					 osso_rpc_cb_f *cb,
+					 gpointer data)
+{
+    return osso_rpc_set_default_cb_f_with_free (osso, cb, data, NULL);
+}
+
 /************************************************************************/
 osso_return_t osso_rpc_unset_cb_f(osso_context_t *osso,
 				  const gchar *service,
@@ -525,16 +682,14 @@ osso_return_t osso_rpc_unset_cb_f(osso_context_t *osso,
 osso_return_t osso_rpc_unset_default_cb_f(osso_context_t *osso,
 					  osso_rpc_cb_f *cb, gpointer data)
 {
-    gchar interface[256] = {0};
+    gchar interface[MAX_IF_LEN] = {0};
     
     if(osso == NULL) return OSSO_INVALID;
     if(cb == NULL) return OSSO_INVALID;
     
-    g_snprintf(interface, 255, "%s.%s", OSSO_BUS_ROOT,
+    g_snprintf(interface, MAX_IF_LEN, OSSO_BUS_ROOT ".%s",
 	       osso->application);
 
-/*    dbus_connection_unregister_object_path(osso->conn, osso->object_path); */
-    
     _rm_cb_f(osso, interface, cb, data);
 
     return OSSO_OK;
@@ -592,34 +747,45 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
     _osso_rpc_t *rpc;
     DBusMessageIter iter;
     GArray *arguments;
-    osso_rpc_t retval = {0};
+    osso_rpc_t retval;
     gint ret;
+    int i;
 
     rpc = (_osso_rpc_t *)data;
     dprint("rpc->func = %p",rpc->func);
     arguments = g_array_new(FALSE, FALSE, sizeof(osso_rpc_t));
 
-    dbus_message_iter_init(msg, &iter);
-    while(TRUE) {
-	osso_rpc_t arg;
+    if (dbus_message_iter_init(msg, &iter)) {
+        while(TRUE) {
+	    osso_rpc_t arg;
 
-	_get_arg(&iter, &arg);
+	    _get_arg(&iter, &arg);
 	
-	dprint("appending value");
-	g_array_append_val(arguments, arg);
+	    dprint("appending value");
+	    g_array_append_val(arguments, arg);
 	
-	if(dbus_message_iter_has_next(&iter))
-	    dbus_message_iter_next(&iter);
-	else
-	    break;
+	    if(dbus_message_iter_has_next(&iter))
+	         dbus_message_iter_next(&iter);
+	    else
+	         break;
+        }
     }
     dprint("calling handler at %p",rpc->func);
-     ret = (rpc->func)(dbus_message_get_interface(msg),
+    retval.type = DBUS_TYPE_INVALID;
+    ret = (rpc->func)(dbus_message_get_interface(msg),
 		      dbus_message_get_member(msg), arguments, rpc->data,
 		      &retval);
+    if (retval.type == DBUS_TYPE_STRING) {
+       dprint("handler returned string '%s'", retval.value.s);
+    }
+
+    for (i = 0; i < arguments->len; i++)
+        osso_rpc_free_val (&g_array_index (arguments, osso_rpc_t, i));
+    g_array_free(arguments, TRUE);
 
     if(ret == OSSO_INVALID) {
-	g_array_free(arguments, TRUE);
+	if (rpc->retval_free)
+	    rpc->retval_free (&retval);
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
     
@@ -630,9 +796,9 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
 	    dprint("callback set retval.type to %d",retval.type);
 	    reply = dbus_message_new_method_return(msg);
 	    if(reply != NULL) {
-/*		dbus_message_append_args(reply, retval.type,
-					 retval.value.i,
-					 DBUS_TYPE_INVALID); */
+                if (retval.type == DBUS_TYPE_STRING) {
+                    dprint("string is still '%s'", retval.value.s);
+                }
 		_append_arg(reply, &retval);
 	    }
 	}
@@ -644,7 +810,7 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
 	    reply = dbus_message_new_error(msg, err_name, retval.value.s);
 	}
 	if(reply != NULL) {
-	    int serial;
+	    dbus_uint32_t serial;
 	    dprint("sending message to '%s'",
 		   dbus_message_get_destination(reply));
 	    
@@ -653,15 +819,18 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
 	    dbus_message_unref(reply);
 	}
     }
-    g_array_free(arguments, TRUE);
+    if (rpc->retval_free)
+        rpc->retval_free (&retval);
     return DBUS_HANDLER_RESULT_HANDLED;
 }
 
 /************************************************************************/
 static void _append_args(DBusMessage *msg, int type, va_list var_args)
 {
+    dprint("");
     while(type != DBUS_TYPE_INVALID) {
 	gchar *s;
+	const char *empty_str = "";
 	gint32 i;
 	guint32 u;
 	gdouble d;
@@ -670,31 +839,29 @@ static void _append_args(DBusMessage *msg, int type, va_list var_args)
 	switch(type) {
 	  case DBUS_TYPE_INT32:
 	    i = va_arg(var_args, gint32);
-	    dbus_message_append_args(msg, type, i, DBUS_TYPE_INVALID);
+	    dprint("appending DBUS_TYPE_INT32 %d", i);
+	    dbus_message_append_args(msg, type, &i, DBUS_TYPE_INVALID);
 	    break;
 	  case DBUS_TYPE_UINT32:
 	    u = va_arg(var_args, guint32);
-	    dbus_message_append_args(msg, type, u, DBUS_TYPE_INVALID);
+	    dbus_message_append_args(msg, type, &u, DBUS_TYPE_INVALID);
 	    break;
 	  case DBUS_TYPE_BOOLEAN:
 	    b = va_arg(var_args, gboolean);
-	    dbus_message_append_args(msg, type, b, DBUS_TYPE_INVALID);
+	    dbus_message_append_args(msg, type, &b, DBUS_TYPE_INVALID);
 	    break;
 	  case DBUS_TYPE_DOUBLE:
 	    d = va_arg(var_args, gdouble);
-	    dbus_message_append_args(msg, type, d, DBUS_TYPE_INVALID);
+	    dbus_message_append_args(msg, type, &d, DBUS_TYPE_INVALID);
 	    break;
 	  case DBUS_TYPE_STRING:
+	    dprint("DBUS_TYPE_STRING");
 	    s = va_arg(var_args, gchar *);
 	    if(s == NULL)
-		dbus_message_append_args(msg, DBUS_TYPE_NIL,
+		dbus_message_append_args(msg, type, &empty_str,
 					 DBUS_TYPE_INVALID);
 	    else
-		dbus_message_append_args(msg, type, s, DBUS_TYPE_INVALID);
-	    break;
-	  case DBUS_TYPE_NIL:	    
-		dbus_message_append_args(msg, DBUS_TYPE_NIL,
-					 DBUS_TYPE_INVALID);
+		dbus_message_append_args(msg, type, &s, DBUS_TYPE_INVALID);
 	    break;
 	  default:
 	    break;	    
@@ -709,41 +876,40 @@ static void _append_args(DBusMessage *msg, int type, va_list var_args)
 
 static void _append_arg(DBusMessage *msg, osso_rpc_t *arg)
 {
+    const char* empty_str = "";
     if(arg == NULL) return;
     switch(arg->type) {
       case DBUS_TYPE_INVALID:
 	break;
       case DBUS_TYPE_INT32:
 	dprint("Appending INT32:%d",arg->value.i);
-	dbus_message_append_args(msg, DBUS_TYPE_INT32, arg->value.i,
+	dbus_message_append_args(msg, DBUS_TYPE_INT32, &arg->value.i,
 				 DBUS_TYPE_INVALID);
 	break;
       case DBUS_TYPE_UINT32:
 	dprint("Appending UINT32:%d",arg->value.u);
-	dbus_message_append_args(msg, DBUS_TYPE_UINT32, arg->value.u,
+	dbus_message_append_args(msg, DBUS_TYPE_UINT32, &arg->value.u,
 				 DBUS_TYPE_INVALID);
 	break;
       case DBUS_TYPE_BOOLEAN:
 	dprint("Appending BOOL:%s",arg->value.b?"TRUE":"FALSE");
-	dbus_message_append_args(msg, DBUS_TYPE_BOOLEAN, arg->value.b,
+	dbus_message_append_args(msg, DBUS_TYPE_BOOLEAN, &arg->value.b,
 				 DBUS_TYPE_INVALID);
 	break;
       case DBUS_TYPE_DOUBLE:
 	dprint("Appending DOUBLE:%f",arg->value.d);
-	dbus_message_append_args(msg, DBUS_TYPE_DOUBLE, arg->value.d,
+	dbus_message_append_args(msg, DBUS_TYPE_DOUBLE, &arg->value.d,
 				 DBUS_TYPE_INVALID);
 	break;
       case DBUS_TYPE_STRING:
 	dprint("Appending STR:'%s'",arg->value.s);
 	if(arg->value.s == NULL)
-	    dbus_message_append_args(msg, DBUS_TYPE_NIL, DBUS_TYPE_INVALID);
+	    dbus_message_append_args(msg, DBUS_TYPE_STRING, &empty_str,
+                                     DBUS_TYPE_INVALID);
 	else
-	    dbus_message_append_args(msg, DBUS_TYPE_STRING, arg->value.s,
+	    dbus_message_append_args(msg, DBUS_TYPE_STRING, &arg->value.s,
 				     DBUS_TYPE_INVALID);
 	break;
-      case DBUS_TYPE_NIL:
-	    dbus_message_append_args(msg, DBUS_TYPE_NIL, DBUS_TYPE_INVALID);
-	break;
       default:
 	break;	    
     }
@@ -758,36 +924,35 @@ static void _get_arg(DBusMessageIter *iter, osso_rpc_t *retval)
     dprint("");
 
     retval->type = dbus_message_iter_get_arg_type(iter);
+    dprint("before switch");
     switch(retval->type) {
       case DBUS_TYPE_INT32:
-	retval->value.i = dbus_message_iter_get_int32(iter);
+        dprint("DBUS_TYPE_INT32");
+	dbus_message_iter_get_basic (iter, &retval->value.i);
 	dprint("got INT32:%d",retval->value.i);
 	break;
       case DBUS_TYPE_UINT32:
-	retval->value.u = dbus_message_iter_get_uint32(iter);
+	dbus_message_iter_get_basic (iter, &retval->value.u);
 	dprint("got UINT32:%u",retval->value.u);
 	break;
       case DBUS_TYPE_BOOLEAN:
-	retval->value.b = dbus_message_iter_get_boolean(iter);
+	dbus_message_iter_get_basic (iter, &retval->value.b);
 	dprint("got BOOLEAN:%s",retval->value.s?"TRUE":"FALSE");
 	break;
       case DBUS_TYPE_DOUBLE:
-	retval->value.d = dbus_message_iter_get_double(iter);
+	dbus_message_iter_get_basic (iter, &retval->value.d);
 	dprint("got DOUBLE:%f",retval->value.d);
 	break;
       case DBUS_TYPE_STRING:
-	str = dbus_message_iter_get_string (iter);
+        dprint("DBUS_TYPE_STRING");
+	dbus_message_iter_get_basic (iter, &str);
 	retval->value.s = g_strdup (str);
-	dbus_free (str);
 	if(retval->value.s == NULL) {
 	    retval->type = DBUS_TYPE_INVALID;
+	    retval->value.i = 0;	    
 	}
 	dprint("got STRING:'%s'",retval->value.s);
 	break;
-      case DBUS_TYPE_NIL:
-	retval->value.s = NULL;	    
-	dprint("got NIL");
-	break;
       default:
 	retval->type = DBUS_TYPE_INVALID;
 	retval->value.i = 0;	    
@@ -805,7 +970,7 @@ static void _async_return_handler(DBusPendingCall *pending, void *data)
     int type;
     rpc = (_osso_rpc_async_t *)data;
     ULOG_INFO_F("At msg return handler");
-    msg = dbus_pending_call_get_reply(pending);
+    msg = dbus_pending_call_steal_reply(pending);
     if(msg == NULL) {
 	g_free(rpc->interface);
 	g_free(rpc->method);
@@ -824,12 +989,21 @@ static void _async_return_handler(DBusPendingCall *pending, void *data)
 	(rpc->func)(rpc->interface, rpc->method, &retval,
 			  rpc->data);
 	dprint("done calling cb");
+	osso_rpc_free_val(&retval);
     }
     else if(type == DBUS_MESSAGE_TYPE_ERROR) {
 	DBusError err;
 	dbus_error_init(&err);
 	dbus_set_error_from_message(&err, msg);
+        dprint("Error return: %s", err.message);
+	retval.type = DBUS_TYPE_STRING;
+	retval.value.s = g_strdup(err.message);
 	dbus_error_free(&err);
+	dprint("calling cb with the error string");
+	(rpc->func)(rpc->interface, rpc->method, &retval,
+			  rpc->data);
+	dprint("done calling cb");
+	osso_rpc_free_val(&retval);
     }
 
     dbus_message_unref(msg);
