@@ -6,8 +6,8 @@
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,34 +22,10 @@
 
 #include "osso-state.h"
 #include "osso-log.h"
+#include "osso-internal.h"
 
-/**
- * This internal function performs a simple validation for the application
- * and version information of the osso_context regarding their validity
- * as components of the filesystem (no slashes, value not NULL etc)
- * @param osso The osso context containing the application name and
- * version information
- * @return TRUE if the context passes the validation, FALSE otherwise.
- */
-
-/* TODO: Move validation to osso_initialization?
- */
-
-/************************************************************************/
-static gboolean _validate(osso_context_t * osso)
-{
-    if ((osso == NULL) || (osso->application  == NULL) ||
-	(osso->version == NULL)) {
-	return FALSE;
-    }
-    if ((strstr(osso->application, "/") != NULL)) {
-	return FALSE;
-    }
-    if ((strstr(osso->version, "/") != NULL)) {
-	return FALSE;
-    }
-    return TRUE;
-}
+static osso_return_t _write_state(const gchar *statefile, osso_state_t *state);
+static osso_return_t _read_state(const gchar *statefile, osso_state_t *state);
 
 static gboolean _validate_state(osso_state_t *state)
 {
@@ -68,13 +44,14 @@ osso_return_t osso_state_write(osso_context_t *osso, osso_state_t *state)
 {
     gchar *path;
     gchar *tmpdir_path = NULL;
+    osso_return_t ret;
 
     if (_validate_state(state) == FALSE)
     {
 	ULOG_ERR_F("NULL state pointer, or state size invalid");
 	return OSSO_INVALID;
     }
-    if (_validate(osso) == FALSE) {
+    if (!validate_osso_context(osso)) {
 	ULOG_ERR_F("appname/version invalid or osso context NULL");
 	return OSSO_INVALID;
     }
@@ -93,21 +70,21 @@ osso_return_t osso_state_write(osso_context_t *osso, osso_state_t *state)
       }
     else 
       {
-	path = g_strconcat(FALLBACK_PREFIX, "/",
+	path = g_strconcat(FALLBACK_PREFIX "/",
 			   osso->application, "/",
 			   osso->version, NULL);
       }
 
     if (path == NULL) {
-	ULOG_ERR_F("Allocation of application/version string failed");
+	ULOG_ERR_F("g_strconcat failed");
 	return OSSO_ERROR;
     }
     
-    _write_state(path, state);
+    ret = _write_state(path, state);
     
-    free(path);
+    g_free(path);
 
-    return OSSO_OK;
+    return ret;
 }
 
 
@@ -122,7 +99,7 @@ osso_return_t osso_state_read(osso_context_t *osso, osso_state_t *state)
 	ULOG_ERR_F("NULL state pointer");
 	return OSSO_INVALID;
     }
-    if (_validate(osso) == FALSE) {
+    if (!validate_osso_context(osso)) {
 	ULOG_ERR_F("appname/version invalid or osso context NULL");
 	return OSSO_INVALID;
     }
@@ -139,7 +116,7 @@ osso_return_t osso_state_read(osso_context_t *osso, osso_state_t *state)
       }
     else
       {
-	path = g_strconcat(FALLBACK_PREFIX, "/",
+	path = g_strconcat(FALLBACK_PREFIX "/",
 			   osso->application, "/",
 			   osso->version, NULL);
       }
@@ -151,7 +128,7 @@ osso_return_t osso_state_read(osso_context_t *osso, osso_state_t *state)
 
     ret = _read_state(path, state);
 
-    free(path);
+    g_free(path);
 
     return ret;
 }
@@ -161,10 +138,10 @@ int osso_state_open_write(osso_context_t *osso)
 {
     gint fd = -1;
     gchar *tmpdir_path  = NULL;
-    gchar *path, *app_path = NULL;
+    gchar *path = NULL, *app_path = NULL;
     struct stat statbuf;
 
-    if (_validate(osso) == FALSE) {
+    if (!validate_osso_context(osso)) {
       ULOG_ERR_F("appname/version invalid or osso context NULL");
       return -1;
     }
@@ -179,7 +156,7 @@ int osso_state_open_write(osso_context_t *osso)
       }
     else
       {
-	path = g_strconcat(FALLBACK_PREFIX, "/",
+	path = g_strconcat(FALLBACK_PREFIX "/",
 			   osso->application, "/",
 			   osso->version, NULL);
       }
@@ -193,7 +170,7 @@ int osso_state_open_write(osso_context_t *osso)
 
     if (tmpdir_path == NULL)
       {
-	app_path = g_strconcat(FALLBACK_PREFIX, "/",
+	app_path = g_strconcat(FALLBACK_PREFIX "/",
 			       osso->application,
 			       NULL);
       }
@@ -203,14 +180,14 @@ int osso_state_open_write(osso_context_t *osso)
     }
     if (app_path == NULL) {
       ULOG_ERR_F("Allocation of application string failed");
-      free(path);
+      g_free(path);
       return -1;
     }
     if (stat(app_path, &statbuf) != -1) {
       if (!S_ISDIR(statbuf.st_mode)) {
 	ULOG_ERR_F("Other type of file instead of app directory");
-	free(app_path);
-	free(path);
+	g_free(app_path);
+	g_free(path);
 	return -1;
       }
     } else {
@@ -220,18 +197,18 @@ int osso_state_open_write(osso_context_t *osso)
 
       if (mkdir(app_path, S_IRUSR | S_IWUSR | S_IXUSR) != 0) {
 	ULOG_ERR_F("Could not create application state directory");
-	free(app_path);
-	free(path);
+	g_free(app_path);
+	g_free(path);
 	return -1;
       }
     }
     
-    free(app_path);
+    g_free(app_path);
     if ((fd = open(path, O_WRONLY | O_CREAT | O_TRUNC,
 		   S_IRUSR | S_IWUSR)) == -1) {
       ULOG_ERR_F("Opening of state file failed");
     }
-    free(path);
+    g_free(path);
     return fd;
 }
 
@@ -243,7 +220,7 @@ int osso_state_open_read(osso_context_t *osso)
     gchar *tmpdir_path  = NULL;
     gchar *path = NULL;
 
-    if (_validate(osso) == FALSE) {
+    if (!validate_osso_context(osso)) {
       ULOG_ERR_F("appname/version invalid or osso context NULL");
       return -1;
     }
@@ -252,7 +229,7 @@ int osso_state_open_read(osso_context_t *osso)
     tmpdir_path = getenv(LOCATION_VAR);
     if (tmpdir_path == NULL)
       {
-	path = g_strconcat(FALLBACK_PREFIX, "/",
+	path = g_strconcat(FALLBACK_PREFIX "/",
 			   osso->application, "/",
 			   osso->version, NULL);
       }
@@ -268,7 +245,7 @@ int osso_state_open_read(osso_context_t *osso)
     if ((fd = open(path, O_RDONLY)) == -1) {
       ULOG_ERR_F("Opening of state file failed");
     }
-    free(path);
+    g_free(path);
     return fd;
 }
 
@@ -297,7 +274,7 @@ static osso_return_t _read_state(const gchar *statefile, osso_state_t *state)
     fd = open(statefile, O_RDONLY);
     if ( fd == -1) {	
 	ret = OSSO_ERROR_NO_STATE;
-	goto _get_state_ret1;
+	goto _get_state_ret2;
     }
     
     if(read(fd, &size, sizeof(guint32))==-1) {
@@ -346,6 +323,7 @@ static osso_return_t _read_state(const gchar *statefile, osso_state_t *state)
     }
     dprint("Read %u bytes out of %u", total_bytes, state->state_size);
 
+    _get_state_ret1:
     do {
 	if (close(fd) == 0) {
 	    break;
@@ -353,13 +331,14 @@ static osso_return_t _read_state(const gchar *statefile, osso_state_t *state)
 	    if(errno != EINTR) {
 		ULOG_ERR_F("Unable to close file '%s': %s",
 			   statefile, strerror(errno));
-		ret = OSSO_ERROR;
+		if(ret == OSSO_OK)
+		    ret = OSSO_ERROR;
 		break;
 	    }
 	}
     }while(1);
 
-    _get_state_ret1:
+    _get_state_ret2:
 #ifdef LIBOSSO_DEBUG	
 	{
 	    guint i, sz;
@@ -428,8 +407,7 @@ static osso_return_t _write_state(const gchar *statefile, osso_state_t *state)
     tempfile = g_strconcat(statefile, ".tmpXXXXXX", NULL);
     if(tempfile == NULL) {
 	ULOG_ERR_F("Unable to allocate memory for tempfile");
-	ret = OSSO_ERROR;
-	goto _set_state_ret0;
+	return OSSO_ERROR;
     }
     fd = g_mkstemp(tempfile);
     if(fd == -1) {
@@ -485,7 +463,6 @@ static osso_return_t _write_state(const gchar *statefile, osso_state_t *state)
 	ret = OSSO_ERROR;
     }
     _set_state_ret1:
-    free(tempfile);	
-    _set_state_ret0:
+    g_free(tempfile);	
     return ret;
 }
