@@ -7,8 +7,8 @@
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -60,8 +60,8 @@ osso_return_t osso_time_set_notification_cb(osso_context_t *osso, osso_time_cb_f
   ot->handler = cb;
   ot->data = data;
   ot->name = g_strdup(SIG_NAME);
-  _msg_handler_set_cb_f(osso, TIME_INTERFACE,
-			_time_handler, ot, FALSE);
+  _msg_handler_set_cb_f_free_data(osso, TIME_INTERFACE,
+                                  _time_handler, ot, FALSE);
   dbus_connection_flush(osso->sys_conn);
   ULOG_DEBUG_F("The callback for filter should now be set.");
   return OSSO_OK;
@@ -70,8 +70,9 @@ osso_return_t osso_time_set_notification_cb(osso_context_t *osso, osso_time_cb_f
 
 /************************************************************************/
 
-/* TODO: The real format and number of the arguments the external
-         command will take; the current list is just an example    */
+/* TODO: This function should just send the notification about time
+ *       change, not try to set any time (that would need root privileges
+ *       or some DBus service) */
 
 osso_return_t osso_time_set(osso_context_t *osso, time_t new_time)
 {
