@@ -217,6 +217,7 @@ int raising_signal(void)
     DBusMessage *msg;
     osso_context_t *osso = NULL;
     FILE *f;
+    const dbus_bool_t boolval = TRUE;
     
     unlink(TESTFILE);
     osso = osso_initialize(APP_NAME, APP_VERSION, FALSE, NULL);
@@ -233,7 +234,7 @@ int raising_signal(void)
 	return 0;
     }
     
-    dbus_message_append_args(msg, DBUS_TYPE_BOOLEAN, TRUE,
+    dbus_message_append_args(msg, DBUS_TYPE_BOOLEAN, &boolval,
 			     DBUS_TYPE_INVALID);
     
     dbus_message_set_no_reply(msg, TRUE);
