@@ -97,10 +97,11 @@ testcase *get_tests(void);
 int test_osso_rpc_run_with_invalid_osso( void )
 {
     gint r;
+    const char* s = "print This is a test";
 
     r = osso_rpc_run(NULL,TOP_SERVICE, TOP_OBJECT, TOP_IFACE,
 		     LAUNCH_METHOD, NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_INVALID);
 
     if(r == OSSO_INVALID)
 	return 1;
@@ -112,6 +113,7 @@ int test_osso_rpc_run_with_invalid_osso( void )
 int test_osso_rpc_run_with_invalid_service( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -119,7 +121,7 @@ int test_osso_rpc_run_with_invalid_service( void )
     
     r = osso_rpc_run(osso, NULL, TOP_OBJECT, TOP_IFACE,
 		     LAUNCH_METHOD, NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_INVALID);
 
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
@@ -132,6 +134,7 @@ int test_osso_rpc_run_with_invalid_service( void )
 int test_osso_rpc_run_with_invalid_object( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -139,7 +142,7 @@ int test_osso_rpc_run_with_invalid_object( void )
     
     r = osso_rpc_run(osso, TOP_SERVICE, NULL, TOP_IFACE,
 		     LAUNCH_METHOD, NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_INVALID);
 
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
@@ -152,6 +155,7 @@ int test_osso_rpc_run_with_invalid_object( void )
 int test_osso_rpc_run_with_invalid_if( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -159,7 +163,7 @@ int test_osso_rpc_run_with_invalid_if( void )
     
     r = osso_rpc_run(osso, TOP_SERVICE, TOP_OBJECT, NULL,
 		     LAUNCH_METHOD,NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_INVALID);
 
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
@@ -172,6 +176,7 @@ int test_osso_rpc_run_with_invalid_if( void )
 int test_osso_rpc_run_with_invalid_method( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -179,7 +184,7 @@ int test_osso_rpc_run_with_invalid_method( void )
     
     r = osso_rpc_run(osso, TOP_SERVICE, TOP_OBJECT, TOP_IFACE,
 		     NULL, NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_INVALID);
 
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
@@ -212,6 +217,7 @@ int test_osso_rpc_run_without_args( void )
 int test_osso_rpc_run( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -219,7 +225,7 @@ int test_osso_rpc_run( void )
     
     r = osso_rpc_run(osso, TOP_SERVICE, TOP_OBJECT, TOP_IFACE, 
 		     LAUNCH_METHOD, NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_INVALID);
 
     dprint("osso_rpc_run returned %d, expected %d",r,OSSO_OK);
     osso_deinitialize(osso);
@@ -233,6 +239,10 @@ int test_osso_rpc_run( void )
 int test_osso_rpc_run_multiple_args( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
+    const char* s2 = "another simple string";
+    const char* s3 = "one more";
+    const char* s4 = "and even more";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -240,10 +250,10 @@ int test_osso_rpc_run_multiple_args( void )
     
     r = osso_rpc_run(osso, TOP_SERVICE, TOP_OBJECT, TOP_IFACE, 
 		     LAUNCH_METHOD, NULL, DBUS_TYPE_STRING,
-		     "print This is a test", DBUS_TYPE_STRING,
-		     "another simple string",DBUS_TYPE_STRING,
-		     "one more",DBUS_TYPE_STRING,
-		     "and even more",DBUS_TYPE_INVALID);
+		     &s, DBUS_TYPE_STRING,
+		     &s2, DBUS_TYPE_STRING,
+		     &s3, DBUS_TYPE_STRING,
+		     &s4, DBUS_TYPE_INVALID);
 
     dprint("osso_rpc_run returned %d, expected %d",r,OSSO_OK);
     osso_deinitialize(osso);
@@ -357,6 +367,7 @@ int test_osso_rpc_run_and_return_default( void )
     gint r;
     GSource *to;
     struct foo foobar;
+    dbus_int32_t intval = -12;
 
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
     assert(osso != NULL);
@@ -372,13 +383,13 @@ int test_osso_rpc_run_and_return_default( void )
 
     r = osso_rpc_run_with_defaults(osso, TEST_APP_NAME, "method",
 				   &(foobar.rpc), DBUS_TYPE_INT32,
-				   -12, DBUS_TYPE_INVALID);
+				   &intval, DBUS_TYPE_INVALID);
     dprint("Starting loop...");
     fflush(stdout);
     fflush(stderr);
-/*    g_main_loop_run(foobar.loop);
+    g_main_loop_run(foobar.loop);
     dprint("I got out of that sucking loop!");
-*/    g_free(foobar.loop);
+    g_free(foobar.loop);
     fflush(stdout);
     fflush(stderr);
     osso_deinitialize(osso);
@@ -688,11 +699,14 @@ int test_sending_all_types( void )
     struct bar foobar;
 
     foobar.osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
-    assert(foobar.osso != NULL);
+    if (foobar.osso == NULL) {
+        dprint("could not init Libosso");
+        return 0;
+    }
     
     foobar.count = 0;
     foobar.ret = OSSO_OK;
-    foobar.type = DBUS_TYPE_NIL;
+    foobar.type = DBUS_TYPE_INT32;
 
     foobar.loop = g_main_loop_new(NULL, FALSE);
     to = g_timeout_source_new(500);
@@ -729,26 +743,6 @@ gboolean rpc_run_ret_cb2(gpointer data)
     fflush(stderr);
     fflush(stdout);
     switch(foobar->type) {
-      case DBUS_TYPE_NIL:
-	dprint("Sending:");
-	foobar->send.type = DBUS_TYPE_NIL;
-	foobar->send.value.s = NULL;
-	_print(&foobar->send);
-	
-	foobar->count++;
-	foobar->type=DBUS_TYPE_INT32;
-	r = osso_rpc_run_with_defaults(foobar->osso, TEST_APP_NAME,
-                                       "echo",&(foobar->retval),
-				       foobar->send.type, DBUS_TYPE_INVALID);
-	assert(r == OSSO_OK);
-	if(_compare(&foobar->retval,&foobar->send)) {
-	    dprint("retval:");
-	    _print(&foobar->retval);	
-	}
-	else {
-	    goto error_1;
-	}
-	break;
       case DBUS_TYPE_INT32:
 	foobar->send.type = foobar->type;
 	foobar->send.value.i = -346;
@@ -902,9 +896,6 @@ void _print(osso_rpc_t *rpc)
       case DBUS_TYPE_STRING:
 	dprint("STRING:'%s'",rpc->value.s);
 	break;
-      case DBUS_TYPE_NIL:
-	dprint("NIL");
-	break;
       default:
 	dprint("UNKNOWN(%c):%x",rpc->type,rpc->value.u);
 	break;	    
@@ -916,12 +907,6 @@ gboolean _compare(osso_rpc_t *a, osso_rpc_t *b)
 {
     dprint("comparing"); _print(a); dprint("to"); _print(b);
     
-    if((a->type == DBUS_TYPE_NIL) && (b->type == DBUS_TYPE_STRING) &&
-       (b->value.s == NULL))
-	return TRUE;
-    else if((b->type == DBUS_TYPE_NIL) && (a->type == DBUS_TYPE_STRING) &&
-       (a->value.s == NULL))
-	return TRUE;
     if(a->type != b->type)
 	return FALSE;
 
@@ -931,9 +916,6 @@ gboolean _compare(osso_rpc_t *a, osso_rpc_t *b)
       case DBUS_TYPE_INVALID:
 	dprint("Invalid type matches anything");
 	return TRUE;
-      case DBUS_TYPE_NIL:
-	dprint("NIL has no value");
-	return TRUE;
       case DBUS_TYPE_INT32:
 	dprint("Integer type");
 	return (a->value.i == b->value.i);
@@ -964,10 +946,11 @@ gboolean _compare(osso_rpc_t *a, osso_rpc_t *b)
 int test_osso_rpc_async_run_with_invalid_osso( void )
 {
     gint r;
+    const char* s = "print This is a test";
     
     r = osso_rpc_async_run(NULL,TOP_SERVICE, TOP_OBJECT, TOP_IFACE,
 			   LAUNCH_METHOD, NULL, NULL, DBUS_TYPE_STRING,
-			   "print This is a test", DBUS_TYPE_INVALID);
+			   &s, DBUS_TYPE_INVALID);
     if(r == OSSO_INVALID)
 	return 1;
     else
@@ -978,6 +961,7 @@ int test_osso_rpc_async_run_with_invalid_osso( void )
 int test_osso_rpc_async_run_with_invalid_service( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -985,7 +969,7 @@ int test_osso_rpc_async_run_with_invalid_service( void )
     
     r = osso_rpc_async_run(osso, NULL, TOP_OBJECT, TOP_IFACE,
 			   LAUNCH_METHOD, NULL, NULL, DBUS_TYPE_STRING,
-			   "print This is a test", DBUS_TYPE_INVALID);
+			   &s, DBUS_TYPE_INVALID);
      osso_deinitialize(osso);
     if(r == OSSO_INVALID)
 	return 1;
@@ -997,6 +981,7 @@ int test_osso_rpc_async_run_with_invalid_service( void )
 int test_osso_rpc_async_run_with_invalid_object( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -1004,7 +989,7 @@ int test_osso_rpc_async_run_with_invalid_object( void )
     
     r = osso_rpc_async_run(osso, TOP_SERVICE, NULL, TOP_IFACE,
 			   LAUNCH_METHOD, NULL, NULL, DBUS_TYPE_STRING,
-			   "print This is a test", DBUS_TYPE_INVALID);
+			   &s, DBUS_TYPE_INVALID);
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
 	return 1;
@@ -1016,6 +1001,7 @@ int test_osso_rpc_async_run_with_invalid_object( void )
 int test_osso_rpc_async_run_with_invalid_if( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -1023,7 +1009,7 @@ int test_osso_rpc_async_run_with_invalid_if( void )
     
     r = osso_rpc_async_run(osso, TOP_SERVICE, TOP_OBJECT, NULL,
 			   LAUNCH_METHOD, NULL, NULL, DBUS_TYPE_STRING,
-			   "print This is a test", DBUS_TYPE_INVALID);
+			   &s, DBUS_TYPE_INVALID);
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
 	return 1;
@@ -1035,6 +1021,7 @@ int test_osso_rpc_async_run_with_invalid_if( void )
 int test_osso_rpc_async_run_with_invalid_method( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -1042,7 +1029,7 @@ int test_osso_rpc_async_run_with_invalid_method( void )
     
     r = osso_rpc_async_run(osso, TOP_SERVICE, TOP_OBJECT, TOP_IFACE,
 			   NULL, NULL, NULL, DBUS_TYPE_STRING,
-			   "print This is a test", DBUS_TYPE_INVALID);
+			   &s, DBUS_TYPE_INVALID);
     osso_deinitialize(osso);
     if(r == OSSO_INVALID)
 	return 1;
@@ -1074,6 +1061,7 @@ int test_osso_rpc_async_run_without_args( void )
 int test_osso_rpc_async_run( void )
 {
     osso_context_t *osso;
+    const char* s = "print This is a test";
     gint r;
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
 
@@ -1081,7 +1069,7 @@ int test_osso_rpc_async_run( void )
     
     r = osso_rpc_async_run(osso, TOP_SERVICE, TOP_OBJECT, TOP_IFACE,
 			   LAUNCH_METHOD, NULL, NULL, DBUS_TYPE_STRING,
-			   "print This is a test", DBUS_TYPE_INVALID);
+			   &s, DBUS_TYPE_INVALID);
 
     osso_deinitialize(osso);
     
@@ -1096,6 +1084,7 @@ int test_osso_rpc_async_run_and_return( void )
     osso_context_t *osso;
     gint r;
     struct foo foobar;
+    dbus_int32_t intval = 903;
 
     osso = osso_initialize(APP_NAME, APP_VER, FALSE, NULL);
     assert(osso != NULL);
@@ -1111,7 +1100,7 @@ int test_osso_rpc_async_run_and_return( void )
 
     r = osso_rpc_async_run(osso, TEST_SERVICE, TEST_OBJECT, TEST_IFACE, 
 			   "echo", async_ret_handler, &foobar,
-			   DBUS_TYPE_INT32, 903, DBUS_TYPE_INVALID);
+			   DBUS_TYPE_INT32, intval, DBUS_TYPE_INVALID);
 
     sleep(2);
     dprint("Starting loop...");
@@ -1152,6 +1141,7 @@ void async_ret_handler(const gchar *interface, const gchar *method,
 	_print(retval);
 	_print(&(foobar->rpc));
 	if(!_compare(&(foobar->rpc), retval)) {
+	    dprint("retval was not what was expected");
 	    foobar->ret = 0;
 	}
 	else {
