@@ -112,8 +112,8 @@ int open_statefile_without_appdata_w(void)
     if (osso == NULL) {
         return 0;
     }
-    osso->application = NULL;
-    osso->version = NULL;
+    osso->application[0] = '\0';
+    osso->version[0] = '\0';
     fd = osso_state_open_write(osso);
     osso_deinitialize(osso);
     if (fd == -1)
@@ -131,8 +131,8 @@ int open_statefile_without_appdata_r(void)
     if (osso == NULL) {
         return 0;
     }
-    osso->application = NULL;
-    osso->version = NULL;
+    osso->application[0] = '\0';
+    osso->version[0] = '\0';
     fd = osso_state_open_read(osso);
     osso_deinitialize(osso);
     if (fd == -1)
@@ -172,8 +172,8 @@ int open_statefile_with_illegal_appname_w(void)
 {
     osso_context_t osso;
     gint fd;
-    osso.application = APP_ILLEGALNAME;
-    osso.version = APP_VER;
+    strcpy(osso.application, APP_ILLEGALNAME);
+    strcpy(osso.version, APP_VER);
 
     fd = osso_state_open_write(&osso);
 
@@ -188,8 +188,8 @@ int open_statefile_with_illegal_appname_r(void)
 {
     osso_context_t osso;
     gint fd;
-    osso.application = APP_ILLEGALNAME;
-    osso.version = APP_VER;
+    strcpy(&osso.application[0], APP_ILLEGALNAME);
+    strcpy(&osso.version[0], APP_VER);
 
     fd = osso_state_open_read(&osso);
 
@@ -204,8 +204,8 @@ int open_statefile_with_illegal_appversion_w(void)
 {
     osso_context_t osso;
     gint fd;
-    osso.application = APP_NAME;
-    osso.version = APP_ILLEGALVER;
+    strcpy(&osso.application[0], APP_NAME);
+    strcpy(&osso.version[0], APP_ILLEGALVER);
 
     fd = osso_state_open_write(&osso);
     if (fd == -1) {
@@ -219,8 +219,8 @@ int open_statefile_with_illegal_appversion_r(void)
 {
     osso_context_t osso;
     gint fd;
-    osso.application = APP_NAME;
-    osso.version = APP_ILLEGALVER;
+    strcpy(&osso.application[0], APP_NAME);
+    strcpy(&osso.version[0], APP_ILLEGALVER);
 
     fd = osso_state_open_read(&osso);
     if (fd == -1) {
